'''
Tests against remindme's migration handler
'''

import unittest
import migrations


class Test_Migrations(unittest.TestCase):
    '''Tests against the Migrations handler.'''

    def test_detect_versions(self):
        pass



'''
RemindMe
~~~~~~~~
A Command Line Application that reminds you of somethings you may end
up forgetting. Just add a `remindme` and in the future you could just
make RemindMe show you the content.

Licensed under the MIT License. For Open Source Initiative (OSI)

Contribute to the Project at https://github.com/GochoMugo/remindme
'''


from .config import __version__
from .cli import run


__all__ = ['__version__', 'run', ]



'''
Handles migrations from one version to another version.
'''

import os
import sqlite3
import sys
import remindme


CURRENT_VERSION = None
NEW_VERSION = None

console = remindme.utils.Console("migration")
db_file = remindme.config.PATHS["db_file"]


def detect_versions():
    '''Detects the current version installed.'''
    global CURRENT_VERSION
    global NEW_VERSION
    cwd = os.getcwd()
    sys.path.remove(cwd)
    old_mod = sys.modules["remindme"]
    del sys.modules["remindme"]
    try:
        import remindme
        CURRENT_VERSION = remindme.__version__
        console.info("currently installed version: {0}".format(CURRENT_VERSION))
    except:
        console.error("failed to detect currently installed version")
        pass
    sys.path.append(cwd)
    sys.modules["remindme"] = old_mod
    import remindme # we have to re-import here!!!
    NEW_VERSION = remindme.__version__
    console.info("new version to install: {0}".format(NEW_VERSION))


def migrate():
    '''Handles invoking migrations.'''
    console.info("process migrations")
    global CURRENT_VERSION 
    OLD_VERSION = CURRENT_VERSION
    if CURRENT_VERSION == "0.2.1" or CURRENT_VERSION is None:
        CURRENT_VERSION = migrate_1() # 1st ever migration
    if CURRENT_VERSION is None:
        sys.exit(1)
    if OLD_VERSION != CURRENT_VERSION: # there was change/migration
        return migrate()
    console.success("finished migrating")


def migrate_1():
    '''Migrates from 0.2.1 to 0.3.0

    Renames database column, `keywords`, to `title`.
    See "http://stackoverflow.com/questions/805363/how-do-i-rename-a-column-in
    -a-sqlite-database-table" for more information.
    '''
    console.info("migrating from 0.2.1 to 0.3.0")
    sql_rename_orig_tbl = "ALTER TABLE remindmes RENAME TO tmp_remindmes;"
    sql_create_tbl = "CREATE TABLE remindmes(title, content);"
    sql_copy_old_tbl = '''
        INSERT INTO remindmes(title, content) SELECT *
        FROM tmp_remindmes;
    '''
    sql_drop_orig_tbl = "DROP TABLE tmp_remindmes;"
    try:
        db = sqlite3.connect(db_file)
        cursor = db.cursor()
        cursor.execute(sql_rename_orig_tbl)
        cursor.execute(sql_create_tbl)
        cursor.execute(sql_copy_old_tbl)
        cursor.execute(sql_drop_orig_tbl)
        db.commit()
        console.success("success migrating from 0.2.1 to 0.3.0")
        return "0.3.0"
    except Exception as err:
        db.rollback()
        console.error("error migrating")
        console.error(err)
        return None


def start():
    '''Start migrations.'''
    detect_versions()
    migrate()


if __name__ == "__main__":
    start()



'''
Command-line runner for Application
'''

import argparse
import sys
from . import config
from . import models
from . import utils


# start-up activities
console = utils.Console("runner")
repository = models.RemindmeRepository(config.PATHS["db_file"])


def arg_parser():
    '''Argument Parser.'''
    parser = argparse.ArgumentParser(
        description='Reminds you of something you knew before',
        epilog="See LICENSE at {0}".format(config.LICENSE)
    )
    parser.add_argument('keywords',
                        metavar='TITLE', nargs='*',
                        help='Title for RemindMe')
    parser.add_argument('-l', '--list',
                        action='store_true',
                        help='list all RemindMe titles')
    parser.add_argument('-a', '--add',
                        metavar='title',
                        dest='add', nargs='+',
                        help='add new RemindMe')
    parser.add_argument('-i', '--in',
                        metavar='title',
                        dest='in', nargs='+',
                        help='pipe-in input for a new remindme')
    parser.add_argument('-r', '--remove',
                        metavar='title',
                        dest='remove', nargs='+',
                        help='remove a RemindMe')
    parser.add_argument('-Ra', '--remove-all',
                        action='store_true',
                        help='remove all RemindMes')
    parser.add_argument('-v', '--version',
                        action='version',
                        version='%(prog)s {0}'.format(config.__version__))

    args = parser.parse_args()
    args = vars(args)
    return args


def run():
    '''Run the command-line runner.'''
    args = arg_parser()

    if args['list']:
        if args['keywords']:
            # searching using a phrase
            phrase = ' '.join(args['keywords'])
            remindmes = repository.find(lambda r: r.get_title().startswith(phrase))
        else:
            remindmes = repository.get_remindmes()
        titles = [r.get_title() for r in remindmes]
        titles.sort()
        num = len(titles)
        console.success('Found {0} remindmes'.format(num))
        if num == 0:
            return
        number = 0
        display_content = ""
        for title in titles:
          number += 1
          display_content = ''.join([display_content, '%-2d - %s\n' % (number, title)])
        console.raw(display_content)
        return

    if args['add']:
        title = ' '.join(args['add'])
        results = repository.find_by_title(title)
        if results:
            console.error("A Remindme already has that title")
            return
        message = "Enter what you remember now"
        content = console.get_long_input(message)

        if content is "":
            console.error("We have nothing to save!")
            return

        if repository.create_remindme(title, content):
            console.success('RemindMe will remind you next time.')
        else:
            console.error('RemindMe failed to get that in memory.')

    if args['in']:
        title = ' '.join(args['in'])
        content = sys.stdin.read().strip()
        if content is '':
            console.error('RemindMe got no data!')
        else:
            if repository.create_remindme(title, content):
                console.success('RemindMe will remind you next time')
            else:
                console.error('RemindMe failed to get that in memory.\n\
Maybe there is already another RemindMe with the same keyword.')

    if args['remove']:
        title = ' '.join(args['remove'])
        remindme = repository.find_by_title(title)
        if remindme and remindme.delete():
            console.success('remindme successfully removed')
        else:
            console.error('can NOT remove that. Check if the remindme \
really exists with me.')

    if args['remove_all']:
        confirm = console.get_input("remove All Remindmes(yes/NO)")
        if confirm.strip().lower() != "yes":
             return console.error("removal cancelled")
        if repository.remove_remindmes():
            console.success('removed all of them')
        else:
            console.error('failed to remove all')

    if args['keywords']:
        title = ' '.join(args['keywords'])
        remindme = repository.find_by_title(title)
        if remindme:
            console.success('Reminding you:')
            lines = remindme.get_content().split("\n")
            number = 0
            for line in lines:
                number += 1
                console.raw("%-2d %s\n" % (number, line))
        else:
            console.error('I too can\'t remember that')


if __name__ == '__main__':
    run()



import sys
from . import config


class Console:
    template = "{color}[{title}]: {message}" + config.COLORS["reset"]

    def __init__(self, title):
        self.title = title

    def raw(self, message):
        sys.stdout.write(message)
        sys.stdout.flush()

    def __write(self, _message, color_code, newline):
        message = Console.template.format(title=self.title,
            color=config.COLORS[color_code], message=_message)
        message += "\n" if newline is True else ""
        self.raw(message)

    def log(self, message, newline=True):
        self.__write(message, "reset", newline)
        return self

    def info(self, message, newline=True):
        self.__write(message, "info", newline)
        return self

    def error(self, message, newline=True):
        self.__write(message, "error", newline)
        return self

    def success(self, message, newline=True):
        self.__write(message, "success", newline)
        return self

    def __input(self):
        if config.PY2:
            return raw_input("> ")
        else:
            return input("> ")

    def get_input(self, question):
        self.info(question + "?")
        return self.__input()

    def get_long_input(self, question):
        self.info(question + "?")
        user_input = []
        while 1:
            try:
                words = self.__input()
                if words == ':end':
                    break
                user_input.append(words)
            except KeyboardInterrupt:
                break
        return '\n'.join(user_input)



import os
import sys
import colorama


# project metadata
__version__ = "0.3.0"
LICENSE = "https://github.com/GochoMugo/remindme/blob/master/LICENSE"


# paths
PATHS = {}
PATHS["home"] = os.path.expanduser("~")
PATHS["db_file"] = os.path.join(PATHS["home"], ".remindme.db")


# colors
colorama.init()
COLORS = {}
COLORS["default"] = colorama.Fore.WHITE
COLORS["error"] = colorama.Fore.RED
COLORS["info"] = colorama.Fore.MAGENTA
COLORS["reset"] = colorama.Style.RESET_ALL
COLORS["success"] = colorama.Fore.GREEN


# python version
PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3
